/**
  ******************************************************************************
  * @file    ili9325_lcd_gpio_driver.c
  * @author  WaveShare
  * @version V1.1
  * @date    19-December-2016
  * @brief   ILI9325 LCD GPIO Driver.
  *           This file provides firmware functions to drive the Liquide crystal 
  *           display (LCD) using General Purpose Input/Output (GPIO) peripheral, 
  *           which are often used to drive LCDs without Flexible Static Memory 
  *           Controller (FSMC).
  *
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT(c) 2016 Waveshare </center></h2>
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of Waveshare nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************  
  */ 
/* Includes ------------------------------------------------------------------*/
#include "./ili9325_lcd_gpio_driver.h"
#include "./fonts.h"

/* Private define ------------------------------------------------------------*/

//#define LCD_REG              (*((volatile unsigned short *) 0x6F000000)) /* RS = 0 */
//#define LCD_RAM              (*((volatile unsigned short *) 0x6F010000)) /* RS = 1 */

/* Private variables ---------------------------------------------------------*/
static uint8_t LCD_Type;

/* Private define ------------------------------------------------------------*/
#define  ILI9320    0  /* 0x9320 */
#define  ILI9325    1  /* 0x9325 */
#define  ILI9328    2  /* 0x9328 */
#define  ILI9331    3  /* 0x9331 */
#define  SSD1298    4  /* 0x8999 */
#define  SSD1289    5  /* 0x8989 */
#define  ST7781     6  /* 0x7783 */
#define  LGDP4531   7  /* 0x4531 */
#define  SPFD5408B  8  /* 0x5408 */
#define  R61505U    9  /* 0x1505 0x0505 */     
#define  HX8347D    10 /* 0x0047 */
#define  HX8347A    11 /* 0x0047 */  
#define  LGDP4535   12 /* 0x4535 */  
#define  SSD2119    13 /* 3.5 LCD 0x9919 */

/*******************************************************************************
* Function Name  : LCD_SelectReg
* Description    : 
* Parameter      : - index:
* Return         : None
*******************************************************************************/
void LCD_SelectReg(uint16_t index)
{
  LCD_RS_RESET();
  LCD_RD_SET();
  LCD_GPIO->ODR = index;   /* GPIO_Write(LCD_GPIO, index); */
  LCD_WR_RESET();
  LCD_WR_SET();
}

/*******************************************************************************
* Function Name  : LCD_WriteData
* Description    : 
* Parameter      : - data:
* Return         : None
*******************************************************************************/
void LCD_WriteData(uint16_t data)
{
  LCD_RS_SET();
  LCD_GPIO->ODR = data;   /* GPIO_Write(LCD_GPIO, data); */
  LCD_WR_RESET();
  LCD_WR_SET();
}

/*******************************************************************************
* Function Name  : LCD_ReadData
* Description    : 
* Parameter      : None
* Return         : 
*******************************************************************************/
uint16_t LCD_ReadData(void)
{
  uint16_t value;

  LCD_RS_SET();
  LCD_WR_SET();
  LCD_RD_RESET();

  LCD_GPIO->CRH = 0x44444444;    /* Reset GPIO [15:8] */
  LCD_GPIO->CRL = 0x44444444;    /* Reset GPIO [7:0] */

  value = LCD_GPIO->IDR;

  LCD_GPIO->CRH = 0x33333333;    /* Set GPIO [15:8]: Output mode, max speed 50 MHz. */
  LCD_GPIO->CRL = 0x33333333;    /* Set GPIO [15:8]: Output mode, max speed 50 MHz. */

  LCD_RD_SET();

  return value;
}

/*******************************************************************************
* Function Name  : LCD_WriteReg
* Description    : Writes to the selected LCD register.
* Parameter      : - LCD_Reg: address of the selected register.
*                  - LCD_RegValue: value to write to the selected register.
* Return         : None
*******************************************************************************/
void LCD_WriteReg(uint16_t LCD_Reg, uint16_t LCD_RegValue)
{ 
  /* Write 16-bit Index, then Write Reg */  
  LCD_CS_RESET();
  LCD_SelectReg(LCD_Reg); 
          
  /* Write 16-bit Reg */
  LCD_WriteData(LCD_RegValue);  
  LCD_CS_SET(); 
}

/*******************************************************************************
* Function Name  : LCD_WriteReg
* Description    : Reads the selected LCD Register.
* Parameter      : None
* Return         : LCD Register Value.
*******************************************************************************/
uint16_t LCD_ReadReg(uint16_t LCD_Reg)
{
  uint16_t LCD_RAM;
  
  /* Write 16-bit Index (then Read Reg) */
  LCD_CS_RESET();
  LCD_SelectReg(LCD_Reg);
  /* Read 16-bit Reg */
  LCD_RAM = LCD_ReadData();        
  LCD_CS_SET();
  return LCD_RAM;
}

static void LCD_SetCursor(uint16_t Xpos, uint16_t Ypos)
{uint16_t temp;
  #if (DISP_ORIENTATION == 0)
    
  #elif (DISP_ORIENTATION == 90)
	Ypos = MAX_Y - 1 - Ypos;
    temp = Ypos;
    Ypos = Xpos;
    Xpos=temp;        
  #elif (DISP_ORIENTATION == 180)
    Xpos = MAX_X - 1 - Xpos; 
    Ypos = MAX_Y - 1 - Ypos; 
  #elif (DISP_ORIENTATION == 270)
    temp = Ypos;
    Ypos = Xpos;
    Xpos = MAX_Y - 1 - temp;
  #endif


  if (LCD_Type == SSD1289)
  {
    LCD_WriteReg(0x004e, Xpos);      
    LCD_WriteReg(0x004f, Ypos);  
  }
  else //9325
  {     
    LCD_WriteReg(0x0020, Xpos);    
    LCD_WriteReg(0x0021, Ypos);    
    //LCD_SelectReg(0x0022);
  }  
  
}

/*******************************************************************************
* Function Name  : LCD_Initializtion
* Description    : SSD1963 Resets
* Parameter      : None
* Return         : None
*******************************************************************************/
void LCD_Initializtion(void)
{
  uint16_t DeviceCode;

  delay_ms(100);
  LCD_RESET_GPIO_Port->BSRR = LCD_RESET_Pin;

  /* Enable back light */
  BL_PWM_GPIO_Port->BSRR = BL_PWM_Pin;
  delay_ms(100);
  /* Set MN(multipliers) of PLL, VCO = crystal freq * (N+1) */
  /* PLL freq = VCO/M with 250MHz < VCO < 800MHz */
  /* The max PLL freq is around 120MHz. To obtain 120MHz as the PLL freq */

  DeviceCode = LCD_ReadReg(0x0000);    /* Read LCD ID  */  

  if(DeviceCode == 0x9325 || DeviceCode == 0x9328)  
  {
    LCD_Type = ILI9325;
    LCD_WriteReg(0xE5, 0x78F0); /* set SRAM internal timing */
    LCD_WriteReg(0x01, 0x0100); /* set Driver Output Control */
    LCD_WriteReg(0x02, 0x0700); /* set 1 line inversion */
    LCD_WriteReg(0x03, 0x1030); /* set GRAM write direction and BGR=1 */
    LCD_WriteReg(0x04, 0x0000); /* Resize register */
    LCD_WriteReg(0x08, 0x0207); /* set the back porch and front porch */
    LCD_WriteReg(0x09, 0x0000); /* set non-display area refresh cycle ISC[3:0] */
    LCD_WriteReg(0x0A, 0x0000); /* FMARK function */
    LCD_WriteReg(0x0C, 0x0000); /* RGB interface setting */
    LCD_WriteReg(0x0D, 0x0000); /* Frame marker Position */
    LCD_WriteReg(0x0F, 0x0000); /* RGB interface polarity */
    /*************Power On sequence ****************/
    LCD_WriteReg(0x10, 0x0000); /* SAP, BT[3:0], AP, DSTB, SLP, STB */
    LCD_WriteReg(0x11, 0x0007); /* DC1[2:0], DC0[2:0], VC[2:0] */
    LCD_WriteReg(0x12, 0x0000); /* VREG1OUT voltage */
    LCD_WriteReg(0x13, 0x0000); /* VDV[4:0] for VCOM amplitude */
    LCD_WriteReg(0x07, 0x0001);
    delay_ms(200); 
    /* Dis-charge capacitor power voltage */
    LCD_WriteReg(0x10, 0x1090); /* SAP, BT[3:0], AP, DSTB, SLP, STB */
    LCD_WriteReg(0x11, 0x0227); /* Set DC1[2:0], DC0[2:0], VC[2:0] */
    delay_ms(50);               /* Delay 50ms */
    LCD_WriteReg(0x12, 0x001F); 
    delay_ms(50);               /* Delay 50ms */
    LCD_WriteReg(0x13, 0x1500); /* VDV[4:0] for VCOM amplitude */
    LCD_WriteReg(0x29, 0x0027); /* 04 VCM[5:0] for VCOMH */
    LCD_WriteReg(0x2B, 0x000D); /* Set Frame Rate */
    delay_ms(50);               /* Delay 50ms */
    LCD_WriteReg(0x20, 0x0000); /* GRAM horizontal Address */
    LCD_WriteReg(0x21, 0x0000); /* GRAM Vertical Address */
    /* ----------- Adjust the Gamma Curve ---------- */
    LCD_WriteReg(0x30, 0x0000);
    LCD_WriteReg(0x31, 0x0707);
    LCD_WriteReg(0x32, 0x0307);
    LCD_WriteReg(0x35, 0x0200);
    LCD_WriteReg(0x36, 0x0008);
    LCD_WriteReg(0x37, 0x0004);
    LCD_WriteReg(0x38, 0x0000);
    LCD_WriteReg(0x39, 0x0707);
    LCD_WriteReg(0x3C, 0x0002);
    LCD_WriteReg(0x3D, 0x1D04);
    /* ------------------ Set GRAM area --------------- */
    LCD_WriteReg(0x50, 0x0000); /* Horizontal GRAM Start Address */
    LCD_WriteReg(0x51, 0x00EF); /* Horizontal GRAM End Address */
    LCD_WriteReg(0x52, 0x0000); /* Vertical GRAM Start Address */
    LCD_WriteReg(0x53, 0x013F); /* Vertical GRAM Start Address */
    LCD_WriteReg(0x60, 0xA700); /* Gate Scan Line */
    LCD_WriteReg(0x61, 0x0001); /* NDL, VLE, REV */
    LCD_WriteReg(0x6A, 0x0000); /* set scrolling line */
    /* -------------- Partial Display Control --------- */
    LCD_WriteReg(0x80, 0x0000);
    LCD_WriteReg(0x81, 0x0000);
    LCD_WriteReg(0x82, 0x0000);
    LCD_WriteReg(0x83, 0x0000);
    LCD_WriteReg(0x84, 0x0000);
    LCD_WriteReg(0x85, 0x0000);
    /* -------------- Panel Control ------------------- */
    LCD_WriteReg(0x90, 0x0010);
    LCD_WriteReg(0x92, 0x0600);
    LCD_WriteReg(0x07, 0x0133); /* 262K color and display ON */
  }
  else if(DeviceCode == 0x9320 || DeviceCode == 0x9300)
  {
    LCD_Type = ILI9320;
    LCD_WriteReg(0x00, 0x0000);
    LCD_WriteReg(0x01, 0x0100);  /* Driver Output Contral */
    LCD_WriteReg(0x02, 0x0700);  /* LCD Driver Waveform Contral */
    LCD_WriteReg(0x03, 0x1018);  /* Entry Mode Set */
    
    LCD_WriteReg(0x04, 0x0000);  /* Scalling Contral */
    LCD_WriteReg(0x08, 0x0202);  /* Display Contral */
    LCD_WriteReg(0x09, 0x0000);  /* Display Contral 3.(0x0000) */
    LCD_WriteReg(0x0a, 0x0000);  /* Frame Cycle Contal.(0x0000) */
    LCD_WriteReg(0x0c, (1<<0));  /* Extern Display Interface Contral */
    LCD_WriteReg(0x0d, 0x0000);  /* Frame Maker Position */
    LCD_WriteReg(0x0f, 0x0000);  /* Extern Display Interface Contral 2. */
    
    delay_ms(100);  /* delay 100 ms */    
    LCD_WriteReg(0x07, 0x0101);  /* Display Contral */
    delay_ms(100);  /* delay 100 ms */    
  
    LCD_WriteReg(0x10, (1<<12)|(0<<8)|(1<<7)|(1<<6)|(0<<4));  /* Power Control 1.(0x16b0)  */
    LCD_WriteReg(0x11, 0x0007);                /* Power Control 2 */
    LCD_WriteReg(0x12, (1<<8)|(1<<4)|(0<<0));        /* Power Control 3.(0x0138)  */
    LCD_WriteReg(0x13, 0x0b00);                /* Power Control 4 */
    LCD_WriteReg(0x29, 0x0000);                /* Power Control 7 */
    
    LCD_WriteReg(0x2b, (1<<14)|(1<<4));
      
    LCD_WriteReg(0x50, 0);       /* Set X Start */
    LCD_WriteReg(0x51, 239);      /* Set X End */
    LCD_WriteReg(0x52, 0);      /* Set Y Start */
    LCD_WriteReg(0x53, 319);      /* Set Y End */
    
    LCD_WriteReg(0x60, 0x2700);  /* Driver Output Control */
    LCD_WriteReg(0x61, 0x0001);  /* Driver Output Control */
    LCD_WriteReg(0x6a, 0x0000);  /* Vertical Srcoll Control */
    
    LCD_WriteReg(0x80, 0x0000);  /* Display Position? Partial Display 1 */
    LCD_WriteReg(0x81, 0x0000);  /* RAM Address Start? Partial Display 1 */
    LCD_WriteReg(0x82, 0x0000);  /* RAM Address End-Partial Display 1 */
    LCD_WriteReg(0x83, 0x0000);  /* Displsy Position? Partial Display 2 */
    LCD_WriteReg(0x84, 0x0000);  /* RAM Address Start? Partial Display 2 */
    LCD_WriteReg(0x85, 0x0000);  /* RAM Address End? Partial Display 2 */
    
    LCD_WriteReg(0x90, (0<<7)|(16<<0));  /* Frame Cycle Contral.(0x0013)  */
    LCD_WriteReg(0x92, 0x0000);  /* Panel Interface Contral 2.(0x0000) */
    LCD_WriteReg(0x93, 0x0001);  /* Panel Interface Contral 3. */
    LCD_WriteReg(0x95, 0x0110);  /* Frame Cycle Contral.(0x0110)  */
    LCD_WriteReg(0x97, (0<<8));  
    LCD_WriteReg(0x98, 0x0000);  /* Frame Cycle Contral */
  
    LCD_WriteReg(0x07, 0x0173);
  }
  else if(DeviceCode == 0x9331)
  {
    LCD_Type = ILI9331;
    LCD_WriteReg(0x00E7, 0x1014);
    LCD_WriteReg(0x0001, 0x0100);   /* set SS and SM bit */
    LCD_WriteReg(0x0002, 0x0200);   /* set 1 line inversion */
    LCD_WriteReg(0x0003, 0x1030);   /* set GRAM write direction and BGR=1 */
    LCD_WriteReg(0x0008, 0x0202);   /* set the back porch and front porch */
    LCD_WriteReg(0x0009, 0x0000);   /* set non-display area refresh cycle ISC[3:0] */
    LCD_WriteReg(0x000A, 0x0000);   /* FMARK function */
    LCD_WriteReg(0x000C, 0x0000);   /* RGB interface setting */
    LCD_WriteReg(0x000D, 0x0000);   /* Frame marker Position */
    LCD_WriteReg(0x000F, 0x0000);   /* RGB interface polarity */
    /* Power On sequence */
    LCD_WriteReg(0x0010, 0x0000);   /* SAP, BT[3:0], AP, DSTB, SLP, STB  */
    LCD_WriteReg(0x0011, 0x0007);   /* DC1[2:0], DC0[2:0], VC[2:0] */
    LCD_WriteReg(0x0012, 0x0000);   /* VREG1OUT voltage  */
    LCD_WriteReg(0x0013, 0x0000);   /* VDV[4:0] for VCOM amplitude */
    delay_ms(200);  /* delay 200 ms */    
    LCD_WriteReg(0x0010, 0x1690);   /* SAP, BT[3:0], AP, DSTB, SLP, STB  */
    LCD_WriteReg(0x0011, 0x0227);   /* DC1[2:0], DC0[2:0], VC[2:0] */
    delay_ms(50);  /* delay 50 ms */    
    LCD_WriteReg(0x0012, 0x000C);   /* Internal reference voltage= Vci  */
    delay_ms(50);  /* delay 50 ms */    
    LCD_WriteReg(0x0013, 0x0800);   /* Set VDV[4:0] for VCOM amplitude */
    LCD_WriteReg(0x0029, 0x0011);   /* Set VCM[5:0] for VCOMH */
    LCD_WriteReg(0x002B, 0x000B);   /* Set Frame Rate */
    delay_ms(50);  /* delay 50 ms */    
    LCD_WriteReg(0x0020, 0x0000);   /* GRAM horizontal Address */
    LCD_WriteReg(0x0021, 0x0000);   /* GRAM Vertical Address */
    /* Adjust the Gamma Curve */
    LCD_WriteReg(0x0030, 0x0000);
    LCD_WriteReg(0x0031, 0x0106);
    LCD_WriteReg(0x0032, 0x0000);
    LCD_WriteReg(0x0035, 0x0204);
    LCD_WriteReg(0x0036, 0x160A);
    LCD_WriteReg(0x0037, 0x0707);
    LCD_WriteReg(0x0038, 0x0106);
    LCD_WriteReg(0x0039, 0x0707);
    LCD_WriteReg(0x003C, 0x0402);
    LCD_WriteReg(0x003D, 0x0C0F);
    /* Set GRAM area */
    LCD_WriteReg(0x0050, 0x0000);   /* Horizontal GRAM Start Address */
    LCD_WriteReg(0x0051, 0x00EF);   /* Horizontal GRAM End Address */
    LCD_WriteReg(0x0052, 0x0000);   /* Vertical GRAM Start Address */
    LCD_WriteReg(0x0053, 0x013F);   /* Vertical GRAM Start Address */
    LCD_WriteReg(0x0060, 0x2700);   /* Gate Scan Line */
    LCD_WriteReg(0x0061, 0x0001);   /*  NDL, VLE, REV */
    LCD_WriteReg(0x006A, 0x0000);   /* set scrolling line */
    /* Partial Display Control */
    LCD_WriteReg(0x0080, 0x0000);
    LCD_WriteReg(0x0081, 0x0000);
    LCD_WriteReg(0x0082, 0x0000);
    LCD_WriteReg(0x0083, 0x0000);
    LCD_WriteReg(0x0084, 0x0000);
    LCD_WriteReg(0x0085, 0x0000);
    /* Panel Control */
    LCD_WriteReg(0x0090, 0x0010);
    LCD_WriteReg(0x0092, 0x0600);
    LCD_WriteReg(0x0007, 0x0021);    
    delay_ms(50);  /* delay 50 ms */    
    LCD_WriteReg(0x0007, 0x0061);
    delay_ms(50);  /* delay 50 ms */    
    LCD_WriteReg(0x0007, 0x0133);    /* 262K color and display ON */
  }
  else if(DeviceCode == 0x9919)
  {
    LCD_Type = SSD2119;
    /* POWER ON &RESET DISPLAY OFF */
    LCD_WriteReg(0x28, 0x0006);
    LCD_WriteReg(0x00, 0x0001);    
    LCD_WriteReg(0x10, 0x0000);    
    LCD_WriteReg(0x01, 0x72ef);
    LCD_WriteReg(0x02, 0x0600);
    LCD_WriteReg(0x03, 0x6a38);  
    LCD_WriteReg(0x11, 0x6874);
    LCD_WriteReg(0x0f, 0x0000);    /* RAM WRITE DATA MASK */
    LCD_WriteReg(0x0b, 0x5308);    /* RAM WRITE DATA MASK */
    LCD_WriteReg(0x0c, 0x0003);
    LCD_WriteReg(0x0d, 0x000a);
    LCD_WriteReg(0x0e, 0x2e00);  
    LCD_WriteReg(0x1e, 0x00be);
    LCD_WriteReg(0x25, 0x8000);
    LCD_WriteReg(0x26, 0x7800);
    LCD_WriteReg(0x27, 0x0078);
    LCD_WriteReg(0x4e, 0x0000);
    LCD_WriteReg(0x4f, 0x0000);
    LCD_WriteReg(0x12, 0x08d9);
    /* Adjust the Gamma Curve */
    LCD_WriteReg(0x30, 0x0000);
    LCD_WriteReg(0x31, 0x0104);   
    LCD_WriteReg(0x32, 0x0100);  
    LCD_WriteReg(0x33, 0x0305);  
    LCD_WriteReg(0x34, 0x0505);   
    LCD_WriteReg(0x35, 0x0305);  
    LCD_WriteReg(0x36, 0x0707);  
    LCD_WriteReg(0x37, 0x0300);  
    LCD_WriteReg(0x3a, 0x1200);  
    LCD_WriteReg(0x3b, 0x0800);     
    LCD_WriteReg(0x07, 0x0033);
  }
  else if(DeviceCode == 0x1505 || DeviceCode == 0x0505)
  {
    LCD_Type = R61505U;
   /* second release on 3/5  ,luminance is acceptable, water wave appear during camera preview */
    LCD_WriteReg(0x0007, 0x0000);
    delay_ms(50);  /* delay 50 ms */    
    LCD_WriteReg(0x0012, 0x011C);    /* why need to set several times?  */
    LCD_WriteReg(0x00A4, 0x0001);    /* NVM */
    LCD_WriteReg(0x0008, 0x000F);
    LCD_WriteReg(0x000A, 0x0008);
    LCD_WriteReg(0x000D, 0x0008);    
    /* GAMMA CONTROL */
    LCD_WriteReg(0x0030, 0x0707);
    LCD_WriteReg(0x0031, 0x0007); 
    LCD_WriteReg(0x0032, 0x0603); 
    LCD_WriteReg(0x0033, 0x0700); 
    LCD_WriteReg(0x0034, 0x0202); 
    LCD_WriteReg(0x0035, 0x0002); 
    LCD_WriteReg(0x0036, 0x1F0F);
    LCD_WriteReg(0x0037, 0x0707); 
    LCD_WriteReg(0x0038, 0x0000); 
    LCD_WriteReg(0x0039, 0x0000); 
    LCD_WriteReg(0x003A, 0x0707); 
    LCD_WriteReg(0x003B, 0x0000); 
    LCD_WriteReg(0x003C, 0x0007); 
    LCD_WriteReg(0x003D, 0x0000); 
    delay_ms(50);  /* delay 50 ms */    
    LCD_WriteReg(0x0007, 0x0001);
    LCD_WriteReg(0x0017, 0x0001);    /* Power supply startup enable */
    delay_ms(50);  /* delay 50 ms */    
    /* power control */
    LCD_WriteReg(0x0010, 0x17A0); 
    LCD_WriteReg(0x0011, 0x0217);    /* reference voltage VC[2:0]   Vciout = 1.00*Vcivl */
    LCD_WriteReg(0x0012, 0x011E);    /* Vreg1out = Vcilvl*1.80   is it the same as Vgama1out ?  */
    LCD_WriteReg(0x0013, 0x0F00);    /* VDV[4:0]-->VCOM Amplitude VcomL = VcomH - Vcom Ampl */
    LCD_WriteReg(0x002A, 0x0000);  
    LCD_WriteReg(0x0029, 0x000A);    /* Vcomh = VCM1[4:0]*Vreg1out    gate source voltage?? */
    LCD_WriteReg(0x0012, 0x013E);    /* power supply on */
    /* Coordinates Control */
    LCD_WriteReg(0x0050, 0x0000);
    LCD_WriteReg(0x0051, 0x00EF); 
    LCD_WriteReg(0x0052, 0x0000); 
    LCD_WriteReg(0x0053, 0x013F); 
    /* Pannel Image Control */
    LCD_WriteReg(0x0060, 0x2700); 
    LCD_WriteReg(0x0061, 0x0001); 
    LCD_WriteReg(0x006A, 0x0000); 
    LCD_WriteReg(0x0080, 0x0000); 
    /* Partial Image Control */
    LCD_WriteReg(0x0081, 0x0000); 
    LCD_WriteReg(0x0082, 0x0000); 
    LCD_WriteReg(0x0083, 0x0000); 
    LCD_WriteReg(0x0084, 0x0000); 
    LCD_WriteReg(0x0085, 0x0000); 
    /* Panel Interface Control */
    LCD_WriteReg(0x0090, 0x0013);      /* frenqucy */  
    LCD_WriteReg(0x0092, 0x0300); 
    LCD_WriteReg(0x0093, 0x0005); 
    LCD_WriteReg(0x0095, 0x0000); 
    LCD_WriteReg(0x0097, 0x0000); 
    LCD_WriteReg(0x0098, 0x0000); 
   
    LCD_WriteReg(0x0001, 0x0100); 
    LCD_WriteReg(0x0002, 0x0700); 
    LCD_WriteReg(0x0003, 0x1030); 
    LCD_WriteReg(0x0004, 0x0000); 
    LCD_WriteReg(0x000C, 0x0000); 
    LCD_WriteReg(0x000F, 0x0000); 
    LCD_WriteReg(0x0020, 0x0000); 
    LCD_WriteReg(0x0021, 0x0000); 
    LCD_WriteReg(0x0007, 0x0021); 
    delay_ms(200);  /* delay 200 ms */    
    LCD_WriteReg(0x0007, 0x0061); 
    delay_ms(200);  /* delay 200 ms */    
    LCD_WriteReg(0x0007, 0x0173); 
  }               
  else if(DeviceCode == 0x8989)
  {
    LCD_Type = SSD1289;
    LCD_WriteReg(0x0000, 0x0001);    delay_ms(50);   /* Enable LCD Oscillator */
    LCD_WriteReg(0x0003, 0xA8A4);    delay_ms(50);   
    LCD_WriteReg(0x000C, 0x0000);    delay_ms(50);   
    LCD_WriteReg(0x000D, 0x080C);    delay_ms(50);   
    LCD_WriteReg(0x000E, 0x2B00);    delay_ms(50);   
    LCD_WriteReg(0x001E, 0x00B0);    delay_ms(50);   
    LCD_WriteReg(0x0001, 0x2B3F);    delay_ms(50);   /* 320*240 0x2B3F */
    LCD_WriteReg(0x0002, 0x0600);    delay_ms(50);
    LCD_WriteReg(0x0010, 0x0000);    delay_ms(50);
    LCD_WriteReg(0x0011, 0x6070);    delay_ms(50);
    LCD_WriteReg(0x0005, 0x0000);    delay_ms(50);
    LCD_WriteReg(0x0006, 0x0000);    delay_ms(50);
    LCD_WriteReg(0x0016, 0xEF1C);    delay_ms(50);
    LCD_WriteReg(0x0017, 0x0003);    delay_ms(50);
    LCD_WriteReg(0x0007, 0x0133);    delay_ms(50);         
    LCD_WriteReg(0x000B, 0x0000);    delay_ms(50);
    LCD_WriteReg(0x000F, 0x0000);    delay_ms(50);
    LCD_WriteReg(0x0041, 0x0000);    delay_ms(50);
    LCD_WriteReg(0x0042, 0x0000);    delay_ms(50);
    LCD_WriteReg(0x0048, 0x0000);    delay_ms(50);
    LCD_WriteReg(0x0049, 0x013F);    delay_ms(50);
    LCD_WriteReg(0x004A, 0x0000);    delay_ms(50);
    LCD_WriteReg(0x004B, 0x0000);    delay_ms(50);
    LCD_WriteReg(0x0044, 0xEF00);    delay_ms(50);
    LCD_WriteReg(0x0045, 0x0000);    delay_ms(50);
    LCD_WriteReg(0x0046, 0x013F);    delay_ms(50);
    LCD_WriteReg(0x0030, 0x0707);    delay_ms(50);
    LCD_WriteReg(0x0031, 0x0204);    delay_ms(50);
    LCD_WriteReg(0x0032, 0x0204);    delay_ms(50);
    LCD_WriteReg(0x0033, 0x0502);    delay_ms(50);
    LCD_WriteReg(0x0034, 0x0507);    delay_ms(50);
    LCD_WriteReg(0x0035, 0x0204);    delay_ms(50);
    LCD_WriteReg(0x0036, 0x0204);    delay_ms(50);
    LCD_WriteReg(0x0037, 0x0502);    delay_ms(50);
    LCD_WriteReg(0x003A, 0x0302);    delay_ms(50);
    LCD_WriteReg(0x003B, 0x0302);    delay_ms(50);
    LCD_WriteReg(0x0023, 0x0000);    delay_ms(50);
    LCD_WriteReg(0x0024, 0x0000);    delay_ms(50);
    LCD_WriteReg(0x0025, 0x8000);    delay_ms(50);
    LCD_WriteReg(0x004f, 0);
    LCD_WriteReg(0x004e, 0);
    #if (DISP_ORIENTATION == 0)
      LCD_WriteReg(0x0011, 0x6070);
    #elif (DISP_ORIENTATION == 90)
      LCD_WriteReg(0x0011, 0x6058);
    #elif (DISP_ORIENTATION == 180)
      LCD_WriteReg(0x0011, 0x6040);
    #elif (DISP_ORIENTATION == 270)
     LCD_WriteReg(0x0011, 0x6068);
    #endif
  }
  else if(DeviceCode == 0x8999)
  {
    LCD_Type = SSD1298;    
    LCD_WriteReg(0x0028, 0x0006);
    LCD_WriteReg(0x0000, 0x0001);
    LCD_WriteReg(0x0003, 0xaea4);    /* power control 1---line frequency and VHG, VGL voltage */
    LCD_WriteReg(0x000c, 0x0004);    /* power control 2---VCIX2 output voltage */
    LCD_WriteReg(0x000d, 0x000c);    /* power control 3---Vlcd63 voltage */
    LCD_WriteReg(0x000e, 0x2800);    /* power control 4---VCOMA voltage VCOML=VCOMH*0.9475-VCOMA */
    LCD_WriteReg(0x001e, 0x00b5);    /* POWER CONTROL 5---VCOMH voltage */   
    LCD_WriteReg(0x0001, 0x3b3f);     
    LCD_WriteReg(0x0002, 0x0600);
    LCD_WriteReg(0x0010, 0x0000);
    LCD_WriteReg(0x0011, 0x6830);
    LCD_WriteReg(0x0005, 0x0000);
    LCD_WriteReg(0x0006, 0x0000);
    LCD_WriteReg(0x0016, 0xef1c);  
    LCD_WriteReg(0x0007, 0x0033);    /* Display control 1 */
    /* when GON=1 and DTE=0, all gate outputs become VGL */
    /* when GON=1 and DTE=0, all gate outputs become VGH */
    /* non-selected gate wires become VGL */     
    LCD_WriteReg(0x000b, 0x0000);
    LCD_WriteReg(0x000f, 0x0000);
    LCD_WriteReg(0x0041, 0x0000);
    LCD_WriteReg(0x0042, 0x0000);
    LCD_WriteReg(0x0048, 0x0000);
    LCD_WriteReg(0x0049, 0x013f);
    LCD_WriteReg(0x004a, 0x0000);
    LCD_WriteReg(0x004b, 0x0000); 
    LCD_WriteReg(0x0044, 0xef00);  /* Horizontal RAM start and end address */
    LCD_WriteReg(0x0045, 0x0000);  /* Vretical RAM start address */
    LCD_WriteReg(0x0046, 0x013f);  /* Vretical RAM end address */ 
    LCD_WriteReg(0x004e, 0x0000);  /* set GDDRAM x address counter */
    LCD_WriteReg(0x004f, 0x0000);    /* set GDDRAM y address counter */   
    /* y control */
    LCD_WriteReg(0x0030, 0x0707);
    LCD_WriteReg(0x0031, 0x0202);
    LCD_WriteReg(0x0032, 0x0204);
    LCD_WriteReg(0x0033, 0x0502);
    LCD_WriteReg(0x0034, 0x0507);
    LCD_WriteReg(0x0035, 0x0204);
    LCD_WriteReg(0x0036, 0x0204);
    LCD_WriteReg(0x0037, 0x0502);
    LCD_WriteReg(0x003a, 0x0302);
    LCD_WriteReg(0x003b, 0x0302); 
    LCD_WriteReg(0x0023, 0x0000);
    LCD_WriteReg(0x0024, 0x0000);
    LCD_WriteReg(0x0025, 0x8000);
    LCD_WriteReg(0x0026, 0x7000);
    LCD_WriteReg(0x0020, 0xb0eb);
    LCD_WriteReg(0x0027, 0x007c);
  }
  else if(DeviceCode == 0x5408)
  {
    LCD_Type = SPFD5408B;
    
    LCD_WriteReg(0x0001, 0x0100);    /* Driver Output Contral Register */ 
    LCD_WriteReg(0x0002, 0x0700);      /* LCD Driving Waveform Contral */
    LCD_WriteReg(0x0003, 0x1030);
    
    LCD_WriteReg(0x0004, 0x0000);    /* Scalling Control register */
    LCD_WriteReg(0x0008, 0x0207);    /* Display Control 2 */
    LCD_WriteReg(0x0009, 0x0000);    /* Display Control 3 */
    LCD_WriteReg(0x000A, 0x0000);    /* Frame Cycle Control */
    LCD_WriteReg(0x000C, 0x0000);    /* External Display Interface Control 1 */
    LCD_WriteReg(0x000D, 0x0000);      /* Frame Maker Position */
    LCD_WriteReg(0x000F, 0x0000);    /* External Display Interface Control 2 */
    delay_ms(50);
    LCD_WriteReg(0x0007, 0x0101);    /* Display Control */
    delay_ms(50);
    LCD_WriteReg(0x0010, 0x16B0);      /* Power Control 1 */
    LCD_WriteReg(0x0011, 0x0001);      /* Power Control 2 */
    LCD_WriteReg(0x0017, 0x0001);      /* Power Control 3 */
    LCD_WriteReg(0x0012, 0x0138);      /* Power Control 4 */
    LCD_WriteReg(0x0013, 0x0800);      /* Power Control 5 */
    LCD_WriteReg(0x0029, 0x0009);    /* NVM read data 2 */
    LCD_WriteReg(0x002a, 0x0009);    /* NVM read data 3 */
    LCD_WriteReg(0x00a4, 0x0000);  
    LCD_WriteReg(0x0050, 0x0000);
    LCD_WriteReg(0x0051, 0x00EF);
    LCD_WriteReg(0x0052, 0x0000);
    LCD_WriteReg(0x0053, 0x013F);
       
    LCD_WriteReg(0x0060, 0x2700);    /* Driver Output Control */

    LCD_WriteReg(0x0061, 0x0003);    /* Driver Output Control */
    LCD_WriteReg(0x006A, 0x0000);    /* Vertical Scroll Control */
    
    LCD_WriteReg(0x0080, 0x0000);    /* Display Position ?Partial Display 1 */
    LCD_WriteReg(0x0081, 0x0000);    /* RAM Address Start ?Partial Display 1 */
    LCD_WriteReg(0x0082, 0x0000);    /* RAM address End - Partial Display 1 */
    LCD_WriteReg(0x0083, 0x0000);    /* Display Position ?Partial Display 2 */
    LCD_WriteReg(0x0084, 0x0000);    /* RAM Address Start ?Partial Display 2 */
    LCD_WriteReg(0x0085, 0x0000);    /* RAM address End ?Partail Display2 */
    LCD_WriteReg(0x0090, 0x0013);    /* Frame Cycle Control */
    LCD_WriteReg(0x0092, 0x0000);     /* Panel Interface Control 2 */
    LCD_WriteReg(0x0093, 0x0003);    /* Panel Interface control 3 */
    LCD_WriteReg(0x0095, 0x0110);    /* Frame Cycle Control */
    LCD_WriteReg(0x0007, 0x0173);
  }
  else if(DeviceCode == 0x4531)
  {  
    LCD_Type = LGDP4531;
    /* Setup display */
    LCD_WriteReg(0x00, 0x0001);
    LCD_WriteReg(0x10, 0x0628);
    LCD_WriteReg(0x12, 0x0006);
    LCD_WriteReg(0x13, 0x0A32);
    LCD_WriteReg(0x11, 0x0040);
    LCD_WriteReg(0x15, 0x0050);
    LCD_WriteReg(0x12, 0x0016);
    delay_ms(50);
    LCD_WriteReg(0x10, 0x5660);
    delay_ms(50);
    LCD_WriteReg(0x13, 0x2A4E);
    LCD_WriteReg(0x01, 0x0100);
    LCD_WriteReg(0x02, 0x0300);  
    LCD_WriteReg(0x03, 0x1030);    
    LCD_WriteReg(0x08, 0x0202);
    LCD_WriteReg(0x0A, 0x0000);
    LCD_WriteReg(0x30, 0x0000);
    LCD_WriteReg(0x31, 0x0402);
    LCD_WriteReg(0x32, 0x0106);
    LCD_WriteReg(0x33, 0x0700);
    LCD_WriteReg(0x34, 0x0104);
    LCD_WriteReg(0x35, 0x0301);
    LCD_WriteReg(0x36, 0x0707);
    LCD_WriteReg(0x37, 0x0305);
    LCD_WriteReg(0x38, 0x0208);
    LCD_WriteReg(0x39, 0x0F0B);
    delay_ms(50);
    LCD_WriteReg(0x41, 0x0002);
    LCD_WriteReg(0x60, 0x2700);
    LCD_WriteReg(0x61, 0x0001);
    LCD_WriteReg(0x90, 0x0119);
    LCD_WriteReg(0x92, 0x010A);
    LCD_WriteReg(0x93, 0x0004);
    LCD_WriteReg(0xA0, 0x0100);
    delay_ms(50);
    LCD_WriteReg(0x07, 0x0133);
    delay_ms(50);
    LCD_WriteReg(0xA0, 0x0000);
  }
  else if(DeviceCode == 0x4535)
  {  
    LCD_Type = LGDP4535;  
    LCD_WriteReg(0x15, 0x0030);    /* Set the internal vcore voltage */                                              
    LCD_WriteReg(0x9A, 0x0010);    /* Start internal OSC */
    LCD_WriteReg(0x11, 0x0020);    /* set SS and SM bit */
    LCD_WriteReg(0x10, 0x3428);    /* set 1 line inversion */
    LCD_WriteReg(0x12, 0x0002);    /* set GRAM write direction and BGR=1 */ 
    LCD_WriteReg(0x13, 0x1038);    /* Resize register */
    delay_ms(40);                 
    LCD_WriteReg(0x12, 0x0012);    /* set the back porch and front porch */
    delay_ms(40);                 
    LCD_WriteReg(0x10, 0x3420);    /* set non-display area refresh cycle ISC[3:0] */
    LCD_WriteReg(0x13, 0x3045);    /* FMARK function */
    delay_ms(70);                 
    LCD_WriteReg(0x30, 0x0000);    /* RGB interface setting */
    LCD_WriteReg(0x31, 0x0402);    /* Frame marker Position */
    LCD_WriteReg(0x32, 0x0307);    /* RGB interface polarity */
    LCD_WriteReg(0x33, 0x0304);    /* SAP, BT[3:0], AP, DSTB, SLP, STB */
    LCD_WriteReg(0x34, 0x0004);    /* DC1[2:0], DC0[2:0], VC[2:0] */
    LCD_WriteReg(0x35, 0x0401);    /* VREG1OUT voltage */
    LCD_WriteReg(0x36, 0x0707);    /* VDV[4:0] for VCOM amplitude */
    LCD_WriteReg(0x37, 0x0305);    /* SAP, BT[3:0], AP, DSTB, SLP, STB */
    LCD_WriteReg(0x38, 0x0610);    /* DC1[2:0], DC0[2:0], VC[2:0] */
    LCD_WriteReg(0x39, 0x0610);    /* VREG1OUT voltage */
    LCD_WriteReg(0x01, 0x0100);    /* VDV[4:0] for VCOM amplitude */
    LCD_WriteReg(0x02, 0x0300);    /* VCM[4:0] for VCOMH */
    LCD_WriteReg(0x03, 0x1030);    /* GRAM horizontal Address */
    LCD_WriteReg(0x08, 0x0808);    /* GRAM Vertical Address */
    LCD_WriteReg(0x0A, 0x0008);    
    LCD_WriteReg(0x60, 0x2700);    /* Gate Scan Line */
    LCD_WriteReg(0x61, 0x0001);    /* NDL, VLE, REV */
    LCD_WriteReg(0x90, 0x013E);
    LCD_WriteReg(0x92, 0x0100);
    LCD_WriteReg(0x93, 0x0100);
    LCD_WriteReg(0xA0, 0x3000);
    LCD_WriteReg(0xA3, 0x0010);
    LCD_WriteReg(0x07, 0x0001);
    LCD_WriteReg(0x07, 0x0021);
    LCD_WriteReg(0x07, 0x0023);
    LCD_WriteReg(0x07, 0x0033);
    LCD_WriteReg(0x07, 0x0133);
  }          
  else if(DeviceCode == 0x0047)
  {
    LCD_Type = HX8347D;
    /* Start Initial Sequence */
    LCD_WriteReg(0xEA, 0x00);                          
    LCD_WriteReg(0xEB, 0x20);                                                     
    LCD_WriteReg(0xEC, 0x0C);                                                   
    LCD_WriteReg(0xED, 0xC4);                                                    
    LCD_WriteReg(0xE8, 0x40);                                                     
    LCD_WriteReg(0xE9, 0x38);                                                    
    LCD_WriteReg(0xF1, 0x01);                                                    
    LCD_WriteReg(0xF2, 0x10);                                                    
    LCD_WriteReg(0x27, 0xA3);                                                    
    /* GAMMA SETTING */
    LCD_WriteReg(0x40, 0x01);                           
    LCD_WriteReg(0x41, 0x00);                                                   
    LCD_WriteReg(0x42, 0x00);                                                   
    LCD_WriteReg(0x43, 0x10);                                                    
    LCD_WriteReg(0x44, 0x0E);                                                   
    LCD_WriteReg(0x45, 0x24);                                                  
    LCD_WriteReg(0x46, 0x04);                                                  
    LCD_WriteReg(0x47, 0x50);                                                   
    LCD_WriteReg(0x48, 0x02);                                                    
    LCD_WriteReg(0x49, 0x13);                                                  
    LCD_WriteReg(0x4A, 0x19);                                                  
    LCD_WriteReg(0x4B, 0x19);                                                 
    LCD_WriteReg(0x4C, 0x16);                                                 
    LCD_WriteReg(0x50, 0x1B);                                                   
    LCD_WriteReg(0x51, 0x31);                                                    
    LCD_WriteReg(0x52, 0x2F);                                                     
    LCD_WriteReg(0x53, 0x3F);                                                    
    LCD_WriteReg(0x54, 0x3F);                                                     
    LCD_WriteReg(0x55, 0x3E);                                                     
    LCD_WriteReg(0x56, 0x2F);                                                   
    LCD_WriteReg(0x57, 0x7B);                                                     
    LCD_WriteReg(0x58, 0x09);                                                  
    LCD_WriteReg(0x59, 0x06);                                                 
    LCD_WriteReg(0x5A, 0x06);                                                   
    LCD_WriteReg(0x5B, 0x0C);                                                   
    LCD_WriteReg(0x5C, 0x1D);                                                   
    LCD_WriteReg(0x5D, 0xCC);                                                   
    /* Power Voltage Setting */
    LCD_WriteReg(0x1B, 0x18);                                                    
    LCD_WriteReg(0x1A, 0x01);                                                    
    LCD_WriteReg(0x24, 0x15);                                                    
    LCD_WriteReg(0x25, 0x50);                                                    
    LCD_WriteReg(0x23, 0x8B);                                                   
    LCD_WriteReg(0x18, 0x36);                           
    LCD_WriteReg(0x19, 0x01);                                                    
    LCD_WriteReg(0x01, 0x00);                                                   
    LCD_WriteReg(0x1F, 0x88);                                                    
    delay_ms(50);
    LCD_WriteReg(0x1F, 0x80);                                                  
    delay_ms(50);
    LCD_WriteReg(0x1F, 0x90);                                                   
    delay_ms(50);
    LCD_WriteReg(0x1F, 0xD0);                                                   
    delay_ms(50);
    LCD_WriteReg(0x17, 0x05);                                                    
    LCD_WriteReg(0x36, 0x00);                                                    
    LCD_WriteReg(0x28, 0x38);                                                 
    delay_ms(50);
    LCD_WriteReg(0x28, 0x3C);                                                
  }
  else if(DeviceCode == 0x7783)
  {
    LCD_Type = ST7781;
    /* Start Initial Sequence */
    LCD_WriteReg(0x00FF, 0x0001);
    LCD_WriteReg(0x00F3, 0x0008);
    LCD_WriteReg(0x0001, 0x0100);
    LCD_WriteReg(0x0002, 0x0700);
    LCD_WriteReg(0x0003, 0x1030);  
    LCD_WriteReg(0x0008, 0x0302);
    LCD_WriteReg(0x0008, 0x0207);
    LCD_WriteReg(0x0009, 0x0000);
    LCD_WriteReg(0x000A, 0x0000);
    LCD_WriteReg(0x0010, 0x0000);  
    LCD_WriteReg(0x0011, 0x0005);
    LCD_WriteReg(0x0012, 0x0000);
    LCD_WriteReg(0x0013, 0x0000);
    delay_ms(50);
    LCD_WriteReg(0x0010, 0x12B0);
    delay_ms(50);
    LCD_WriteReg(0x0011, 0x0007);
    delay_ms(50);
    LCD_WriteReg(0x0012, 0x008B);
    delay_ms(50);  
    LCD_WriteReg(0x0013, 0x1700);
    delay_ms(50);  
    LCD_WriteReg(0x0029, 0x0022);    
    LCD_WriteReg(0x0030, 0x0000);
    LCD_WriteReg(0x0031, 0x0707);
    LCD_WriteReg(0x0032, 0x0505);
    LCD_WriteReg(0x0035, 0x0107);
    LCD_WriteReg(0x0036, 0x0008);
    LCD_WriteReg(0x0037, 0x0000);
    LCD_WriteReg(0x0038, 0x0202);
    LCD_WriteReg(0x0039, 0x0106);
    LCD_WriteReg(0x003C, 0x0202);
    LCD_WriteReg(0x003D, 0x0408);
    delay_ms(50);        
    LCD_WriteReg(0x0050, 0x0000);    
    LCD_WriteReg(0x0051, 0x00EF);    
    LCD_WriteReg(0x0052, 0x0000);    
    LCD_WriteReg(0x0053, 0x013F);    
    LCD_WriteReg(0x0060, 0xA700);    
    LCD_WriteReg(0x0061, 0x0001);
    LCD_WriteReg(0x0090, 0x0033);        
    LCD_WriteReg(0x002B, 0x000B);    
    LCD_WriteReg(0x0007, 0x0133);
  }
  else  /* special ID */
  {
    DeviceCode = LCD_ReadReg(0x67);
    if(DeviceCode == 0x0047)
    {
      LCD_Type = HX8347A;
      LCD_WriteReg(0x0042, 0x0008);   
      /* Gamma setting */  
      LCD_WriteReg(0x0046, 0x00B4); 
      LCD_WriteReg(0x0047, 0x0043); 
      LCD_WriteReg(0x0048, 0x0013); 
      LCD_WriteReg(0x0049, 0x0047); 
      LCD_WriteReg(0x004A, 0x0014); 
      LCD_WriteReg(0x004B, 0x0036); 
      LCD_WriteReg(0x004C, 0x0003); 
      LCD_WriteReg(0x004D, 0x0046); 
      LCD_WriteReg(0x004E, 0x0005);  
      LCD_WriteReg(0x004F, 0x0010);  
      LCD_WriteReg(0x0050, 0x0008);  
      LCD_WriteReg(0x0051, 0x000a);  
      /* Window Setting */
      LCD_WriteReg(0x0002, 0x0000); 
      LCD_WriteReg(0x0003, 0x0000); 
      LCD_WriteReg(0x0004, 0x0000); 
      LCD_WriteReg(0x0005, 0x00EF); 
      LCD_WriteReg(0x0006, 0x0000); 
      LCD_WriteReg(0x0007, 0x0000); 
      LCD_WriteReg(0x0008, 0x0001); 
      LCD_WriteReg(0x0009, 0x003F); 
      delay_ms(10); 
      LCD_WriteReg(0x0001, 0x0006); 
      LCD_WriteReg(0x0016, 0x00C8);   
      LCD_WriteReg(0x0023, 0x0095); 
      LCD_WriteReg(0x0024, 0x0095); 
      LCD_WriteReg(0x0025, 0x00FF); 
      LCD_WriteReg(0x0027, 0x0002); 
      LCD_WriteReg(0x0028, 0x0002); 
      LCD_WriteReg(0x0029, 0x0002); 
      LCD_WriteReg(0x002A, 0x0002); 
      LCD_WriteReg(0x002C, 0x0002); 
      LCD_WriteReg(0x002D, 0x0002); 
      LCD_WriteReg(0x003A, 0x0001);  
      LCD_WriteReg(0x003B, 0x0001);  
      LCD_WriteReg(0x003C, 0x00F0);    
      LCD_WriteReg(0x003D, 0x0000); 
      delay_ms(20); 
      LCD_WriteReg(0x0035, 0x0038); 
      LCD_WriteReg(0x0036, 0x0078); 
      LCD_WriteReg(0x003E, 0x0038); 
      LCD_WriteReg(0x0040, 0x000F); 
      LCD_WriteReg(0x0041, 0x00F0);  
      LCD_WriteReg(0x0038, 0x0000); 
      /* Power Setting */ 
      LCD_WriteReg(0x0019, 0x0049);  
      LCD_WriteReg(0x0093, 0x000A); 
      delay_ms(10); 
      LCD_WriteReg(0x0020, 0x0020);   
      LCD_WriteReg(0x001D, 0x0003);   
      LCD_WriteReg(0x001E, 0x0000);  
      LCD_WriteReg(0x001F, 0x0009);   
      LCD_WriteReg(0x0044, 0x0053);  
      LCD_WriteReg(0x0045, 0x0010);   
      delay_ms(10);  
      LCD_WriteReg(0x001C, 0x0004);  
      delay_ms(20); 
      LCD_WriteReg(0x0043, 0x0080);    
      delay_ms(5); 
      LCD_WriteReg(0x001B, 0x000a);    
      delay_ms(40);  
      LCD_WriteReg(0x001B, 0x0012);    
      delay_ms(40);   
      /* Display On Setting */ 
      LCD_WriteReg(0x0090, 0x007F); 
      LCD_WriteReg(0x0026, 0x0004); 
      delay_ms(40);  
      LCD_WriteReg(0x0026, 0x0024); 
      LCD_WriteReg(0x0026, 0x002C); 
      delay_ms(40);   
      LCD_WriteReg(0x0070, 0x0008); 
      LCD_WriteReg(0x0026, 0x003C);  
      LCD_WriteReg(0x0057, 0x0002); 
      LCD_WriteReg(0x0055, 0x0000); 
      LCD_WriteReg(0x0057, 0x0000); 
    }   
  }              
    delay_ms(50);   /* delay 50 ms */
}

/*******************************************************************************
* Function Name  : LCD_Clear
* Description    : 
* Parameter      : - Color: Screen Color
* Return         : None
*******************************************************************************/
void LCD_Clear(uint16_t Color)
{
  uint32_t index=0;
  
  if(LCD_Type == HX8347D || LCD_Type == HX8347A)
  {
    LCD_WriteReg(0x02, 0x00);                                                  
    LCD_WriteReg(0x03, 0x00);  
                    
    LCD_WriteReg(0x04, 0x00);                           
    LCD_WriteReg(0x05, 0xEF);  
                     
    LCD_WriteReg(0x06, 0x00);                           
    LCD_WriteReg(0x07, 0x00);    
                   
    LCD_WriteReg(0x08, 0x01);                           
    LCD_WriteReg(0x09, 0x3F);     
  }
  else
  {  
    LCD_SetCursor(0, 0); 
  }  
  LCD_CS_RESET(); 
  LCD_SelectReg(0x0022);
  for(index = 0; index < MAX_X * MAX_Y; index++)
  {
    LCD_WriteData(Color);
  }
  LCD_CS_SET();
}

/******************************************************************************
* Function Name  : LCD_BGR2RGB
* Description    : RRRRRGGGGGGBBBBB convert to BBBBBGGGGGGRRRRR
* Parameter      : RGB color
* Return         : RGB color
* Attention      :
*******************************************************************************/
static uint16_t LCD_BGR2RGB(uint16_t color)
{
  uint16_t  r, g, b, rgb;
  
  b = (color>>0)  & 0x1f;
  g = (color>>5)  & 0x3f;
  r = (color>>11) & 0x1f;
  
  rgb =  (b<<11) + (g<<5) + (r<<0);
  
  return(rgb);
}

/******************************************************************************
* Function Name  : LCD_GetPoint
* Description    : Get color of the point
* Parameter      : - Xpos: Row Coordinate
*                  - Ypos: Line Coordinate 
* Return         : Screen Color
*******************************************************************************/
uint16_t LCD_GetPoint(uint16_t Xpos, uint16_t Ypos)
{
  uint16_t dummy;
  
  LCD_SetCursor(Xpos, Ypos);
  LCD_CS_RESET();
  LCD_SelectReg(0x0022);  
  
  switch(LCD_Type)
  {
    case ST7781:
    case LGDP4531:
    case LGDP4535:
    case SSD1289:
    case SSD1298:
      dummy = LCD_ReadData();
      dummy = LCD_ReadData(); 
      LCD_CS_SET();  
       return  dummy;        
    case HX8347A:
    case HX8347D:
    {
      uint8_t red, green, blue;
      red = LCD_ReadData()>>3;
      green = LCD_ReadData()>>3; 
      blue = LCD_ReadData()>>2; 
      dummy = (green << 11) | (blue << 5) | red;
    }
      LCD_CS_SET();  
      return  dummy;
    default:  /* 0x9320 0x9325 0x9328 0x9331 0x5408 0x1505 0x0505 0x9919 */
      dummy = LCD_ReadData();
      dummy = LCD_ReadData(); 
      LCD_CS_SET();  
      return  LCD_BGR2RGB(dummy);
  }
}
void Delay_ms( uint32_t ms )
{
	static __IO uint32_t TimingDelay;
	uint32_t x, y;
	TimingDelay = ms;

	
	
  for(x = 0; x < ms; x++)
	{
		for(y = 0; y < 1141; y++);
	}
}

/******************************************************************************
* Function Name  : LCD_SetPoint
* Description    : 
* Parameter      : - Xpos: Row Coordinate
*                  - Ypos: Line Coordinate 
* Return         : None
*******************************************************************************/
void LCD_SetPoint(uint16_t Xpos, uint16_t Ypos, uint16_t point)
{
  if(Xpos >= MAX_X || Ypos >= MAX_Y)
  {
    return;
  }
  LCD_SetCursor(Xpos, Ypos);
  LCD_WriteReg(0x0022, point);
}



/******************************************************************************
* Function Name  : LCD_DrawLine
* Description    : Bresenham's line algorithm
* Parameter      : - x0:
*                  - y0:
*                  - x1:
*                  - y1:
*                  - color:
* Return         : None
*******************************************************************************/   
void LCD_Box(uint16_t color,uint16_t x0, uint16_t y0, uint16_t x1, uint16_t y1)
{
   uint16_t x,y;
 for(x=0;x<x1;x++)
	{
		LCD_SetCursor(x0+x,y0+y1);
		LCD_CS_RESET();
		LCD_SelectReg(0x022);
		for(y=0;y<y1;y++)
		LCD_WriteData(color);
		LCD_CS_SET();
	}
}

  /*if(x0 > x1)
  {
    temp = x1;
    x1 = x0;
    x0 = temp;   
  }
  if(y0 > y1)
  {
    temp = y1;
    y1 = y0;
    y0 = temp;   
  }

  dx = x1-x0;
  dy = y1-y0;

  if(dx == 0)
  {
    do
    { 
      LCD_SetPoint(x0, y0, color);
      y0++;
    }
    while(y1 >= y0); 
    return; 
  }
  if(dy == 0)
  {
    do
    {
      LCD_SetPoint(x0, y0, color);
      x0++;
    }
    while(x1 >= x0); 
    return;
  }

  Bresenham's line algorithm  */
  /*if(dx > dy)
  {
    temp = 2 * dy - dx;
    while(x0 != x1)
    {
      LCD_SetPoint(x0, y0, color);
      x0++;
      if(temp > 0)
      {
        y0++;
        temp += 2 * dy - 2 * dx; 
       }
      else         
      {
        temp += 2 * dy;
      }       
    }
    LCD_SetPoint(x0, y0, color);
  }  
  else
  {
    temp = 2 * dx - dy;
    while(y0 != y1)
    {
       LCD_SetPoint(x0, y0, color);     
      y0++;                 
      if(temp > 0)           
      {
        x0++;               
        temp+=2*dy-2*dx; 
      }
      else
      {
        temp += 2 * dy;
      }
    } 
    LCD_SetPoint(x0, y0, color);
  }
}*/

/******************************************************************************
* Function Name  : PutChar
* Description    : 
* Parameter      : - Xpos:
*                  - Ypos:
*                  - ASCI:
*                  - charColor:
*                  - bkColor:
* Return         : None
*******************************************************************************/
void PutChar(uint16_t Xpos, uint16_t Ypos, uint8_t Char, uint16_t charColor, sFONT *font, uint16_t bkColor)
{
  uint16_t x, y;
  uint16_t charOffset = (font->Width % 8) ? (font->Height * (font->Width / 8 + 1)) : (font->Height * font->Width / 8);
  const uint8_t *pDraw = &(font->table)[(Char - 32) * charOffset];
  
  for (y = 0; y < font->Height; y++) 
  {
    for (uint16_t count = 0; count < ((font->Width % 8) ? (font->Width / 8 + 1) : (font->Width / 8)); count++) 
    {
      for (x = 0; x < 8; x++) 
      {
        if((*pDraw >> (7 - x)) & 0x01) 
        {
          LCD_SetPoint(Xpos + x + 8 * count, Ypos + y, charColor);
        } 
        else 
        {
          LCD_SetPoint(Xpos + x + 8 * count, Ypos + y, bkColor);
        }
      }
      pDraw++;
    }
  }
  
}

/******************************************************************************
* Function Name  : GUI_Text
* Description    : 
* Parameter      : - Xpos: 
*                  - Ypos: 
*                  - str:
*                  - charColor:
*                  - bkColor:
* Return         : None
*******************************************************************************/
void GUI_Text(uint16_t Xpos, uint16_t Ypos, uint8_t *str, uint16_t Color, sFONT *font, uint16_t bkColor)
{
  do
  {
    PutChar(Xpos, Ypos, *str++, Color, font, bkColor);    
    if(Xpos < MAX_X - font->Width)
    {
      Xpos += font->Width;
    } 
    else if (Ypos < MAX_Y - font->Height)
    {
      Xpos = 0;
      Ypos += font->Height;
    }   
    else
    {
      Xpos = 0;
      Ypos = 0;
    }    
  }
  while (*str != 0);
}



/*********************************************************************************************************
      END FILE
*********************************************************************************************************/
